#include "MeshLoader.h"
using namespace std;

vector<Node>& MeshLoader::Get_Nodes() {
    return nodes;
}
vector<FiniteElement>& MeshLoader::Get_FiniteElements() {
    return finiteelements;
}
vector<BoundaryFiniteElement>& MeshLoader::Get_BFEs() {
    return bfes;
}
vector<FiniteElement> MeshLoader::Get_FE_by_nodes(int id1, int id2, int id3) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()){
    it = find_if(it, finiteelements.end(), [id1, id2, id3](FiniteElement x) {
        return ((find(x.nodes_id.begin(), x.nodes_id.end(), id1) != x.nodes_id.end()) &&
                (find(x.nodes_id.begin(), x.nodes_id.end(), id2) != x.nodes_id.end()) &&
                (find(x.nodes_id.begin(), x.nodes_id.end(), id3) != x.nodes_id.end())
        );
    });
    if (it != finiteelements.end()) {
        ans.push_back(*it);
        it++;
    }
}
    return ans;
}
vector<FiniteElement> MeshLoader::Get_FE_by_edge(int id1, int id2) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()) {
        it = find_if(it, finiteelements.end(), [id1, id2](FiniteElement x) {
            return ((find(x.nodes_id.begin(), x.nodes_id.end(), id1) != x.nodes_id.end()) &&
                    (find(x.nodes_id.begin(), x.nodes_id.end(), id2) != x.nodes_id.end())
            );
        });
        if (it != finiteelements.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}

vector<Node> MeshLoader::Get_nodes_by_border(int id) {
    vector <Node> ans;
    auto it = bfes.begin();
    while(it != bfes.end()){
        it = find_if(it, bfes.end(), [id](BoundaryFiniteElement x){
            return x.border_id = id;});
        for (const auto& x : it->nodes_id){
            auto node_by_ID = std::find_if(nodes.begin(), nodes.end(), [x](Node n){ return n.id == x; });
            ans.push_back(*node_by_ID);
        }
        it++;
    }
    return ans;
}
vector<FiniteElement>MeshLoader::Get_FE_by_material(int id) {
    vector<FiniteElement> ans;
    auto it = finiteelements.begin();
    while (it != finiteelements.end()) {
        it = find_if(it, finiteelements.end(), [id](FiniteElement x) {
            return x.mat_id == id; });
        if (it != finiteelements.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}
vector <BoundaryFiniteElement>MeshLoader::Get_BFE_by_border(int id) {
    vector<BoundaryFiniteElement> ans;
    auto it = bfes.begin();
    while (it != bfes.end()) {
        it = find_if(it, bfes.end(), [id](BoundaryFiniteElement x) {
            return x.border_id == id; });
        if (it != bfes.end()) {
            ans.push_back(*it);
            it++;
        }
    }
    return ans;
}

void MeshLoader::Insertion(Node node, FiniteElement fe) {
    int p = fe.nodes_id.size() / 2;
    auto it = fe.nodes_id.begin();
    advance(it, p);
    fe.nodes_id.insert(it, node.id);
    nodes.push_back(node);
}
Node MeshLoader::Get_node_by_id(vector <Node> x, int id)
{
    auto it = find_if(x.begin(), x.end(), [&](Node x){
        return x.id == id;
    });
    if (it == x.end())
    {
        Node* t = new Node;
        t->id = -100;
        return *t;
    }
    else
        return *it;
}
vector <vector<Node>> MeshLoader::Get_neighbors() {
    vector <vector<Node>> ans(nodes.size() + 1);
    int i = 0;
    for(auto& itn : nodes)
    {
        for(auto& itfe : finiteelements)
        {
            auto it = find_if(((itfe).nodes_id).begin(), ((itfe).nodes_id).end(), [&] (auto id){
                return id = itn.id;
            });
            if (it != ((itfe).nodes_id).end())
            {
                for(auto& j : itfe.nodes_id)
                {
                    Node tmp = Get_node_by_id(nodes, j);
                    if(tmp.id > 0)
                        ans[i].push_back(tmp);
                }
            }
            i++;
        }
    }
    return ans;
}
void MeshLoader::printNode(const Node &node) {
    std::cout << node;
}

void MeshLoader::printFE(const FiniteElement &fe) {
    std::cout << fe;
}

void MeshLoader::printBFE(const BoundaryFiniteElement &bfe) {
    std::cout << bfe;
}
vector<int> MeshLoader::getBoundaryNodesId() {
    std::vector<int> res;
    for (const auto &it: bfes) {
        std::for_each(it.nodes_id.begin(), it.nodes_id.end(), [&res](int id) {
            res.push_back(id);
        });
    }
    return res;
}


